{% extends "base.html" %}

{% set title = "Riego - Ventilboxen" %}
{% set active_menu = "boxes" %}


{% block content %}
<h1>Items</h1>
<ul>
    {% for item in items %}
    <li>
        <a href="{{url('boxes_item_view', item_id=item['id']) }}">{{ item['name'] }}</a> {{ item['topic'] }}
    </li>
    {% endfor %}
</ul>
<hr>
<p>
    <a href="{{url('boxes_new')}}">New</a>
</p>

{% endblock %}